"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const build_1 = require("@angular/build");
const architect_1 = require("@angular-devkit/architect");
const core_1 = require("@angular-devkit/core");
const rxjs_1 = require("rxjs");
const load_module_1 = require("../load-module");
const load_plugins_1 = require("../load-plugins");
// Path where Angular loads Vite for reference:
// https://github.com/angular/angular-cli/blob/main/packages/angular/build/src/builders/dev-server/vite-server.ts#L183
// Path where we could inject Vite plugins:
// https://github.com/angular/angular-cli/blob/main/packages/angular/build/src/builders/dev-server/vite-server.ts#L890
function executeBuilder(options, context) {
    const buildTarget = (0, architect_1.targetFromTargetString)(options.buildTarget);
    const workspaceRoot = (0, core_1.getSystemPath)((0, core_1.normalize)(context.workspaceRoot));
    return (0, rxjs_1.from)(
    // This will combine all options from the executed and build target
    context.getTargetOptions(buildTarget)).pipe((0, rxjs_1.switchMap)(async (buildOptions) => {
        const tsConfig = node_path_1.default.join(workspaceRoot, buildOptions.tsConfig);
        const middleware = [];
        // Keep middleware order
        for (const middlewarePath of options.middlewares ?? []) {
            middleware.push(await (0, load_module_1.loadModule)(workspaceRoot, middlewarePath, tsConfig));
        }
        const buildPlugins = await (0, load_plugins_1.loadPlugins)(buildOptions.plugins, workspaceRoot, tsConfig);
        const indexHtmlTransformer = buildOptions.indexHtmlTransformer
            ? await (0, load_module_1.loadModule)(workspaceRoot, buildOptions.indexHtmlTransformer, tsConfig)
            : undefined;
        return { middleware, buildPlugins, indexHtmlTransformer };
    }), 
    // TODO: add proper extensions types once release by Angular/build
    (0, rxjs_1.switchMap)((extensions) => {
        return (0, build_1.executeDevServerBuilder)(options, context, extensions);
    }));
}
exports.default = (0, architect_1.createBuilder)(executeBuilder);
//# sourceMappingURL=index.js.map