"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadModule = loadModule;
const tslib_1 = require("tslib");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const load_esm_1 = require("./load-esm");
let lastTsConfig;
async function tsNodeRegister(tsConfig) {
    if (lastTsConfig) {
        return;
    }
    lastTsConfig = tsConfig;
    const tsNode = await import("ts-node");
    tsNode.register({
        project: tsConfig,
        compilerOptions: {
            module: "CommonJS",
            moduleResolution: "Node",
            target: "ES2020",
            noCheck: true,
            esModuleInterop: true,
            allowSyntheticDefaultImports: true,
            resolveJsonModule: true,
            types: ["node"],
        },
    });
    // Thanks to https://github.com/just-jeb/angular-builders/blob/master/packages/common/src/load-module.ts#L31C47-L40C6
    const tsPaths = await import("tsconfig-paths");
    const result = tsPaths.loadConfig(tsConfig);
    if (result.resultType === "success") {
        const { absoluteBaseUrl: baseUrl, paths } = result;
        tsPaths.register({ baseUrl, paths });
    }
}
async function loadModule(workspaceRoot, modulePath, tsConfig) {
    await tsNodeRegister(tsConfig);
    // Possible module import
    let resolvedModulePath = modulePath;
    if (modulePath.startsWith(".")) {
        // Relative file path
        resolvedModulePath = node_path_1.default.join(workspaceRoot, modulePath);
    }
    switch (node_path_1.default.extname(resolvedModulePath)) {
        case ".mjs":
        case ".js":
            return (await (0, load_esm_1.loadEsmModule)(resolvedModulePath)).default;
        case ".cjs":
            return require(resolvedModulePath).default;
        case ".ts":
            return (await (0, load_esm_1.loadEsmModule)(resolvedModulePath)).default;
        default:
            try {
                return require(resolvedModulePath).default;
            }
            catch (error) {
                if (error.code === "ERR_REQUIRE_ESM") {
                    return (await (0, load_esm_1.loadEsmModule)(resolvedModulePath))
                        .default;
                }
                throw error;
            }
    }
}
//# sourceMappingURL=load-module.js.map